###############################################################################
#    Copyright (c) 2016 Salvatore Ventura <salvoventura@gmail.com>
#
#      File: photos.py
#
#    Author: Salvatore Ventura <salvoventura@gmail.com>
#      Date: 07 Dec 2016
#   Purpose: Handle Photos, CuratedPhotos, RandomPhotos, and Photo
#
#  Revision: 1
#   Comment: What's new in revision 1
#
###############################################################################
from .liblogging import logger
from .unpage import UnsplashPage
from .unobject import UnsplashObject


class Photos(UnsplashPage):
    def __init__(self, api_key, url='/photos', **kwargs):
        valid_options = ['page', 'per_page', 'order_by']
        super(Photos, self).__init__(url=url, api_key=api_key, valid_options=valid_options, **kwargs)

    @property
    def entries(self):
        for entry in self.body:
            yield Photo(api_key=self.api_key, source=entry)


class CuratedPhotos(UnsplashPage):
    # TODO: 2019 this endpoint is deprecated
    def __init__(self, api_key, url='/photos/curated', **kwargs):
        logger.warning('This endpoint has been deprecated by the Unsplash API')
        valid_options = ['page', 'per_page', 'order_by']
        super(CuratedPhotos, self).__init__(url=url, api_key=api_key, valid_options=valid_options, **kwargs)

    @property
    def entries(self):
        for entry in self.body:
            yield Photo(api_key=self.api_key, source=entry)


class RandomPhotos(UnsplashPage):
    def __init__(self, api_key, url='/photos/random', **kwargs):
        valid_options = ['collections', 'featured', 'username', 'query', 'orientation', 'count']
        super(RandomPhotos, self).__init__(url=url, api_key=api_key, valid_options=valid_options, **kwargs)

    @property
    def entries(self):
        for entry in self.body:
            yield Photo(api_key=self.api_key, source=entry)


class SinglePhoto(UnsplashPage):
    def __init__(self, api_key, photo_id):
        url = '/photos/{}'.format(photo_id)
        valid_options = ['photo_id']
        super(SinglePhoto, self).__init__(url=url, api_key=api_key, valid_options=valid_options)

    @property
    def entries(self):
        return Photo(api_key=self.api_key, source=self.body)


class Photo(UnsplashObject):
    def __init__(self, api_key, source, **kwargs):
        valid_options = ['w', 'h', 'rect']
        # simplify for source=photoid
        if isinstance(source, str) and not source.startswith(self._api_root):
            # assume it's a photoid
            source = "{}/photos/{}".format(self._api_root, source)
        super(Photo, self).__init__(api_key=api_key, source=source, valid_options=valid_options, **kwargs)

    @property
    def link_html(self):
        return self.links.get('html', None)

    @property
    def stats(self):
        url = '{}/stats'.format(self.url)
        r = self._agent.get(url)
        if r.status_code == 200:
            return r.json()
        else:
            logger.error('Failed stats retrieval for %s with status code %s', url, r.status_code)
            return None

    @property
    def link_download(self):
        return self.links.get('download', None)

    @property
    def link_download_location(self):
        return self.links.get('download_location', None)

    def get_attribution(self, format='txt'):
        """
        Return the standard Unsplash attribution string, which currently is:
            Photo by <firstname> <lastname> on Unsplash
        In html, it should link to the photographer's profile:
            <span>Photo by <a href=<url>><firstname> <lastname></a> on <a href=<url>>Unsplash</a></span>
        :param format: valid values: 'str', 'html'
        :return: string of text or html for attribution
        """
        author_info = self.body.get('user', None)
        if author_info:
            # Strictly extracting information used for the attribution string
            # If we ever expand, we should make a new class
            first_name = author_info.get('first_name', "")
            last_name = author_info.get('last_name', "")
            url = author_info.get('links', {}).get('html', "")
            logger.debug("Photo author {} {}, on Unsplash at {}".format(first_name, last_name, url))
        if format in ('str', 'txt'):
            return "Photo by {} {} on Unsplash".format(first_name, last_name)
        if format == 'html':
            return '<span>Photo by <a href="{}">{} {}</a> on <a href="{}">Unsplash</a></span>'.format(url, first_name,
                                                                                                      last_name, url)



###############################################################################
#    Copyright (c) 2016 Salvatore Ventura <salvoventura@gmail.com>
#
#      File: unpage.py
#
#    Author: Salvatore Ventura <salvoventura@gmail.com>
#      Date: 06 Dec 2016
#   Purpose: Base class for Unsplash API pages
#
#  Revision: 1
#   Comment: What's new in revision 1
#            Any object exposes GET/PUT/POST/DELETE
#               url is optional; if none provided, the current url is used
#               next, previous, last, first *might* be available and usable
#               object itself has no memory
#               GET, next, previous, last, first return a new object
#               POST returns TBD
#               PUT no returns (true/false or error...)
#               DELETE no returns (true, false or error...)
###############################################################################
from .liblogging import logger
from .rest import Rest
from .settings import API_ROOT


class UnsplashPage(object):
    _api_root = API_ROOT

    def __init__(self, url, api_key=None, valid_options=None, **kwargs):
        self.api_key = api_key
        self.valid_options = valid_options

        if url is None:
            raise ValueError('UnsplashPage: url cannot be None')

        self.url = self._sanitized_url(url)
        self.query_parameters = self._sanitized_query_parameters(kwargs)

        self._agent = None                            # potentially move outside init
        self.navigation = None
        self.requests = None

        if api_key is not None:
            self._agent = Rest(api_key=self.api_key)

        if self.url is not None:
            r = self._agent.get(self.url, self.query_parameters)
            if r.status_code == 200:
                self.requests = r
                self._parse_navigation()

    @property
    def body(self):
        if self.requests:
            return self.requests.json()

    @property
    def entries(self):
        # must override in child class
        raise NotImplementedError

    @property
    def header(self):
        if self.requests:
            return self.requests.headers

    @property
    def status_code(self):
        if self.requests:
            return self.requests.status_code

    def set_api_key(self, api_key):
        self.api_key = api_key
        self._agent = Rest(api_key=self.api_key)

    def get(self, url, query_parameters=None):
        return self.__class__(url=url, query_parameters=query_parameters, api_key=self.api_key)

    def put(self, url, body):
        # TODO
        pass

    def post(self, url, body):
        # TODO
        pass

    def delete(self, url):
        # TODO
        pass

    @property
    def link_self(self):
        return self._ret_link('self')

    @property
    def link_next(self):
        return self._ret_link('next')

    @property
    def link_previous(self):
        return self._ret_link('prev')

    @property
    def link_first(self):
        return self._ret_link('first')

    @property
    def link_last(self):
        return self._ret_link('last')

    @property
    def has_next(self):
        return self.link_next is not None

    @property
    def has_previous(self):
        return self.link_previous is not None

    def get_next_page(self):
        return self.__class__(url=self.link_next, api_key=self.api_key)

    def get_previous_page(self):
        return self.__class__(url=self.link_previous, api_key=self.api_key)

    def get_first_page(self):
        return self.__class__(url=self.link_first, api_key=self.api_key)

    def get_last_page(self):
        return self.__class__(url=self.link_last, api_key=self.api_key)

    def _sanitized_url(self, url):
        logger.debug('call _sanitized_url(%s)', url)
        if url.startswith(self._api_root):
            good_url = url
        elif url.startswith('/'):
            good_url = '{}{}'.format(self._api_root, url)
        else:
            good_url = '{}/{}'.format(self._api_root, url)
        logger.debug('     returning %s', good_url)
        return good_url

    def _sanitized_query_parameters(self, kwargs):
        logger.debug('call _sanitized_query_parameters(%s)', kwargs)
        query_params = {}
        for key in kwargs:
            # Raising an exception here: if we are too protective it will hurt debuggability
            if self.valid_options and key not in self.valid_options:
                logger.debug('Invalid parameter %s' % key)
                raise ValueError('Invalid parameter %s' % key)
            query_params[key] = kwargs[key]
        logger.debug('     returning %s', query_params)
        return query_params

    def _parse_navigation(self):
        navigation = {'self': self.url, 'first': None, 'prev': None, 'next': None, 'last': None}
        pagination = self.requests.headers.get('Link', None)
        if pagination:
            for item in pagination.split(','):
                # rel="first", rel="prev", rel="next", rel="last"
                link, rel = [l.strip(' <>') for l in item.split(';')]
                # here rel value is 'rel="next"'
                if rel[5:-1] in navigation.keys():
                    navigation[rel[5:-1]] = link
        self.navigation = navigation

    def _ret_link(self, which):
        if self.navigation:
            return self.navigation.get(which, None)



###############################################################################
#    Copyright (c) 2016 Salvatore Ventura <salvoventura@gmail.com>
#
#      File: test_photos.py
#
#    Author: Salvatore Ventura <salvoventura@gmail.com>
#      Date: 14 Dec 2016
#   Purpose: photos unit tests
#
#  Revision: 1
#   Comment: What's new in revision 1
#            use local resources
#
###############################################################################
import responses
import json
import os
from pyunsplash import PyUnsplash
from pyunsplash.src.settings import API_ROOT

api_key = os.environ.get('APPLICATION_ID', None) or 'DUMMY_APPLICATION_ID'


class TestPhotos:
    # TODO: avoid code duplication
    # Need to workout how to combine responses.activate so as to avoid
    # code duplication, as the testcases are pretty much the same for all

    # TOXINIDIR comes from tox.ini
    root_path = os.environ.get('TRAVIS_BUILD_DIR', None) or os.environ.get('TOXINIDIR', None)

    store_mapping = {'generic': os.sep.join([root_path, 'pyunsplash', 'tests', 'resources', 'resource__photos_page_2.json']),
                     'curated': os.sep.join([root_path, 'pyunsplash', 'tests', 'resources', 'resource__photos_curated_page_2.json']),
                     'random': os.sep.join([root_path, 'pyunsplash', 'tests', 'resources', 'resource__photos_random_count_2.json'])}

    @responses.activate
    def test_photos_generic(self):
        type = 'generic'
        resource_filepath = self.store_mapping[type]
        stored_response = json.loads(open(resource_filepath).read())
        responses.add(
            responses.GET,
            '{}{}'.format(API_ROOT, stored_response.get('url').split('?')[0]),   # cheating on the url, because the class always inits without query params
            json=stored_response.get('body'),
            status=stored_response.get('status_code'),
            content_type='application/json',
            adding_headers=stored_response.get('headers')
        )
        pu_obj = PyUnsplash(api_key=api_key)
        photos = pu_obj.photos(type_=type)
        assert photos.body is not None
        assert photos.header is not None
        assert photos.status_code == 200
        for photo in photos.entries:
            # if any of the fields breaks, then it's a problem
            print(photo.id, photo.link_html, photo.link_download, photo.link_download_location)   # , photo.stats  # TODO: include stats in unit test

    @responses.activate
    def test_photos_curated(self):
        type = 'curated'
        resource_filepath = self.store_mapping[type]
        stored_response = json.loads(open(resource_filepath).read())

        responses.add(
            responses.GET,
            '{}{}'.format(API_ROOT, stored_response.get('url').split('?')[0]),   # cheating on the url, because the class always inits without query params
            json=stored_response.get('body'),
            status=stored_response.get('status_code'),
            content_type='application/json',
            adding_headers=stored_response.get('headers')
        )
        pu_obj = PyUnsplash(api_key=api_key)
        photos = pu_obj.photos(type_=type)
        assert photos.body is not None
        assert photos.header is not None
        assert photos.status_code == 200
        for photo in photos.entries:
            # if any of the fields breaks, then it's a problem
            print(photo.id, photo.link_html, photo.link_download, photo.link_download_location)   # , photo.stats  # TODO: include stats in unit test

    @responses.activate
    def test_photos_random(self):
        type = 'random'
        resource_filepath = self.store_mapping[type]
        stored_response = json.loads(open(resource_filepath).read())

        responses.add(
            responses.GET,
            '{}{}'.format(API_ROOT, stored_response.get('url').split('?')[0]),   # cheating on the url, because the class always inits without query params
            json=stored_response.get('body'),
            status=stored_response.get('status_code'),
            content_type='application/json',
            adding_headers=stored_response.get('headers')
        )
        pu_obj = PyUnsplash(api_key=api_key)
        photos = pu_obj.photos(type_=type)
        assert photos.body is not None
        assert photos.header is not None
        assert photos.status_code == 200
        for photo in photos.entries:
            # if any of the fields breaks, then it's a problem
            print(photo.id, photo.link_html, photo.link_download, photo.link_download_location)   # , photo.stats  # TODO: include stats in unit test
            print(photo.get_attribution())
            print(photo.get_attribution(format='txt'))
            print(photo.get_attribution(format='html'))



###############################################################################
#    Copyright (c) 2016 Salvatore Ventura <salvoventura@gmail.com>
#
#      File: collections.py
#
#    Author: Salvatore Ventura <salvoventura@gmail.com>
#      Date: 06 Dec 2016
#   Purpose: Handle Collections, CuratedCollections, FeaturedCollections, and Collection
#
#  Revision: 1
#   Comment: What's new in revision 1
#
###############################################################################
from .liblogging import logger
from .unpage import UnsplashPage
from .unobject import UnsplashObject
from .photos import Photos


class Collections(UnsplashPage):
    def __init__(self, api_key, url='/collections', **kwargs):
        valid_options = ['page', 'per_page']
        super(Collections, self).__init__(url=url, api_key=api_key, valid_options=valid_options, **kwargs)

    @property
    def entries(self):
        for entry in self.body:
            yield Collection(api_key=self.api_key, source=entry)


class CuratedCollections(UnsplashPage):
    # TODO: 2019 this endpoint is deprecated
    def __init__(self, api_key, url='/collections/curated', **kwargs):
        logger.warning('This endpoint has been deprecated by the Unsplash API')
        valid_options = ['page', 'per_page']
        super(CuratedCollections, self).__init__(url=url, api_key=api_key, valid_options=valid_options, **kwargs)

    @property
    def entries(self):
        for entry in self.body:
            yield Collection(api_key=self.api_key, source=entry)


class FeaturedCollections(UnsplashPage):
    def __init__(self, api_key, url='/collections/featured', **kwargs):
        valid_options = ['page', 'per_page']
        super(FeaturedCollections, self).__init__(url=url, api_key=api_key, valid_options=valid_options, **kwargs)

    @property
    def entries(self):
        for entry in self.body:
            yield Collection(api_key=self.api_key, source=entry)


class Collection(UnsplashObject):
    def __init__(self, api_key, source):
        super(Collection, self).__init__(api_key=api_key, source=source)

    @property
    def title(self):
        return self.body.get('title', None)

    @property
    def description(self):
        return self.body.get('description', None)

    @property
    def user(self):
        return self.body.get('user', None)

    @property
    def link_photos(self):
        return self.links.get('photos')

    @property
    def link_related(self):
        return self.links.get('related')

    @property
    def related(self):
        # Apparently, 'related' doesn't honor parameters
        # TODO: cache the returned object
        url = self.link_related
        return Collections(url=url, api_key=self.api_key)

    def photos(self, **kwargs):
        # TODO: cache the returned object
        url = self.link_photos
        return Photos(url=url, api_key=self.api_key, **kwargs)



###############################################################################
#    Copyright (c) 2020 Salvatore Ventura <salvoventura@gmail.com>
#
#      File: example_search.py
#
#    Author: Salvatore Ventura <salvoventura@gmail.com>
#      Date: 6/1/2020
#   Purpose: Example file for Search
#
#  Revision: 1
#   Comment: What's new in revision 1
#
###############################################################################
import logging
import os
from pyunsplash import PyUnsplash
api_key = os.environ.get('APPLICATION_ID', None) or 'DUMMY_APPLICATION_ID'

# Initialize app logging
logger = logging.getLogger()
logging.basicConfig(filename='app.log', level=logging.DEBUG)

# pyunsplash logger defaults to level logging.ERROR
# If you need to change that, use getLogger/setLevel
# on the module logger, like this:
logging.getLogger(PyUnsplash.logger_name).setLevel(logging.DEBUG)

# instantiate PyUnsplash object
py_un = PyUnsplash(api_key=api_key)

search = py_un.search(type_="photos", query="bear")
while search.has_next:
    print(search.link_next, search.link_last)
    search = search.get_next_page()



###############################################################################
#    Copyright (c) 2016 Salvatore Ventura <salvoventura@gmail.com>
#
#      File: settings.py
#
#    Author: Salvatore Ventura <salvoventura@gmail.com>
#      Date: 07 Dec 2016
#   Purpose: Global repository of common constants and values
#
#  Revision: 1
#   Comment: What's new in revision 1
#
###############################################################################
import logging
API_ROOT = 'https://api.unsplash.com'
LIB_NAME = 'pyunsplash'
LOG_LEVEL = logging.ERROR



###############################################################################
#
#      File: pyunsplash.py
#
#    Author: Salvatore Ventura <salvoventura@gmail.com>
#      Date: 08 Dec 2016
#   Purpose: simplify import statements
#
#   Comment:
#
###############################################################################

"""An open source Python wrapper for the Unsplash REST API"""

from .pyunsplash import PyUnsplash
# TRAVIS_BUILD_NUMBER
# TRAVIS_COMMIT
# TRAVIS_COMMIT_RANGE
# TRAVIS_TAG
__version__ = '1.0.0rc2'
__author__ = 'Salvatore Ventura <salvoventura@gmail.com>'
__license__ = 'MIT'



###############################################################################
#    Copyright (c) 2016 Salvatore Ventura <salvoventura@gmail.com>
#
#      File: search.py
#
#    Author: Salvatore Ventura <salvoventura@gmail.com>
#      Date: 08 Dec 2016
#   Purpose: Handle Search
#
#  Revision: 1
#   Comment: What's new in revision 1
#
###############################################################################
# from .liblogging import logger
from .unpage import UnsplashPage
from .collections import Collection
from .photos import Photo
from .users import User


class Search(UnsplashPage):
    def __init__(self, api_key, url, **kwargs):
        valid_options = ['page', 'per_page', 'query']
        self.url = url
        if url.lower() in ['photos', 'collections', 'users']:
            self.url = url.lower()
            url = '/search/{}'.format(url.lower())
        super(Search, self).__init__(url=url, api_key=api_key, valid_options=valid_options, **kwargs)

    @property
    def entries(self):
        if self.url == 'photos' or self.url.endswith('/photos'):
            class_cast = Photo
        elif self.url == 'collections' or self.url.endswith('/collections'):
            class_cast = Collection
        elif self.url == 'users' or self.url.endswith('/users'):
            class_cast = User
        for entry in self.body.get('results', []):
            yield class_cast(api_key=self.api_key, source=entry)



###############################################################################
#    Copyright (c) 2017 Salvatore Ventura <salvoventura@gmail.com>
#
#      File: liblogging.py
#
#    Author: Salvatore Ventura <salvoventura@gmail.com>
#      Date: 07 Sep 2017
#   Purpose: Centralize module logging
#
#  Revision: 1
#   Comment: At the module level pyunsplash defines a logger called via LIB_NAME
#            called in this case "pyunsplash".
#            As per best practice, it is off by default, and can be tuned directly
#            by using getLogger/setLevel from the application layer.
#            See examples for this.
#
###############################################################################
import logging
from .settings import LIB_NAME, LOG_LEVEL
logger = logging.getLogger(LIB_NAME)
logger.addHandler(logging.NullHandler())
logger.setLevel(LOG_LEVEL)



# from distutils.core import setup
from setuptools import setup

setup(
    name='pyunsplash',
    version='1.0.0rc2',  # beta, v9
    packages=['pyunsplash', 'pyunsplash.src'],
    url='https://github.com/salvoventura/pyunsplash',
    license='MIT',
    author='salvatore ventura',
    author_email='salvoventura@gmail.com',
    description='A Python wrapper for the Unsplash.com REST API',
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 4 - Beta',

        # Indicate who your project is intended for
        'Intended Audience :: Developers',

        # Pick your license as you wish (should match "license" above)
        'License :: OSI Approved :: MIT License',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8'
    ],
    keywords=['unsplash', 'rest', 'api', 'python', 'wrapper', 'development', 'unsplash.com', 'photography'],
    install_requires=['requests'],
    python_requires='>=2.7'
)




###############################################################################
#    Copyright (c) 2016 Salvatore Ventura <salvoventura@gmail.com>
#
#      File: example_iteration.py
#
#    Author: Salvatore Ventura <salvoventura@gmail.com>
#      Date: 14 Dec 2016
#   Purpose: How to get to every photo in every collection
#
#  Revision: 2
#   Comment: What's new in revision 2
#            Add application level logging and interaction with
#            library logging example
#
#   Comment: What's new in revision 1
#            Show the use of the different objects
#            To get every photo, you could just use Photos
#
#
###############################################################################
import logging
import os
from pyunsplash import PyUnsplash
api_key = os.environ.get('APPLICATION_ID', None) or 'DUMMY_APPLICATION_ID'


# Initialize app logging
logger = logging.getLogger()
logging.basicConfig(filename='app.log', level=logging.DEBUG)

# pyunsplash logger defaults to level logging.ERROR
# If you need to change that, use getLogger/setLevel
# on the module logger, like this:
logging.getLogger(PyUnsplash.logger_name).setLevel(logging.DEBUG)


# instantiate PyUnsplash object
py_un = PyUnsplash(api_key=api_key)

# Start with the generic collection, maximize number of items
# note: this will run until all photos of all collections have
#       been visited, unless a connection error occurs.
#       Typically the API hourly limit gets hit during this
#
collections = py_un.collections(per_page=30)
while collections.has_next:
    for collection in collections.entries:
        photos = collection.photos()
        for photo in photos.entries:
            print(collection.title, photo.link_download, photo.get_attribution())

    # no need to specify per_page: will take from original object
    collections = collections.get_next_page()
